/* -------------------------------------------------- */
/* Create list of device specifications for PICkit 2  */
/* from pickit_2.dvs file                             */
/* Author: Rob Hamerling                              */
/* -------------------------------------------------- */

say "Disassembling PICkit2 device table"

basedir  = "c:\program files\microchip\pickit2_1.1.0\pickit 2\"   /* dir */
dvs      = basedir"PICKit 2.dvs"                /* filespec */
list     = "PICKit_2.lst"                       /* output listing */

if stream(list, 'c', 'query exist') \= '' then  /* drop previous */
  '@erase' list

                                                /* records are 75 bytes */
rx = stream(dvs, 'c', 'open read binary reclength 75')
k = 0
do while lines(dvs)
  k = k + 1
  p.k = linein(dvs)
end
p.0 = k

call stream dvs, 'c', 'close'                   /* done with dvs */

say 'Sorting' p.0 'PICs on name'

call sortpic                                    /* sort on name */

call lineout list, '; --------------------------------------------------'
call lineout list, '; Disassembled' dvs
call lineout list, '; Sorted on name'
call lineout list, '; --------------------------------------------------'
call lineout list, ''

do k = 1 to p.0
  call displayPIC k                             /* format and list record */
end

call stream list, 'c', 'close'                  /* done with list */

say "See file" list " for results."

return 0


/* ------------------------------------------------ */
/*  sort array of PIC specs                         */
/* ------------------------------------------------ */
sortPIC: procedure expose p.

do i=1 to p.0 - 1                               /* till one but last */
  namei = substr(p.i, 1, 13)                    /* extract name */
  do j=i+1 to p.0                               /* till last */
    namej = substr(p.j, 1, 13)                  /* extract name */
    if namei > namej then do                    /* highest is shifted right */
      tmp = p.j
      p.j = p.i                                 /* exchange records */
      p.j = tmp
    end
  end
end
return


/* ------------------------------------------------ */
/* procedure to write formatted record to list      */
/* ------------------------------------------------ */
displayPIC: procedure expose list p.

parse arg k .                                   /* index in array */

parse var p.k    name,                          /* extract fields */
              14 family,
              15 id,
              19 progmemsize,
              23 eememsize,
              27 testmemsize,
              31 saveosccal,
              32 savebgbits,
              33 includeunusedbits,
              34 pgmmodeentry,
              35 pgmmode,
              36 nvalue,
              37 pgmtiming,
              38 datatiming,
              39 erasemode,
              40 bandgapmask,
              44 configmask,
              72 cpmask

call lineout list, 'Name              =' name
call lineout list, 'Family            =' family
call lineout list, 'ID                =' c2x(reverse(substr(id,1,2)))
call lineout list, 'ProgMemSize       =' 2 * c2d(reverse(progmemsize)) '  ; bytes'
call lineout list, 'EEMemSize         =' c2d(reverse(eememsize)) '  ;bytes'
call lineout list, 'TestMemSize       =' 2 * c2d(reverse(testmemsize)) '  ; bytes'
call lineout list, 'SaveOSCCAL        =' c2d(saveosccal)
call lineout list, 'SaveBGBits        =' savebgbits
call lineout list, 'IncludeUnusedBits =' includeunusedbits
call lineout list, 'PgmModeEntry      =' pgmmodeentry
call lineout list, 'PgmMode           =' pgmmode
call lineout list, 'nValue            =' c2d(reverse(nvalue))
call lineout list, 'PgmTiming         =' pgmtiming
call lineout list, 'DataTiming        =' datatiming
call lineout list, 'EraseMode         =' erasemode
call lineout list, 'BandGapMask       =' c2x(reverse(substr(bandgapmask,1,2)))
call charout list, 'ConfigMask        ='
do i = 1 to 28 by 4
  call charout list, ' 'c2x(reverse(substr(configmask,i,2)))
end
call lineout list, ''
call lineout list, 'CPMask            =' c2x(reverse(substr(cpmask,1,2)))
call lineout list, ''
return

